var EXPORTED_SYMBOLS = ["initGlobal","TuentiGroups","TuentiTheme"]  //Note: this globals will never be added to global scope

/*
 *This is the global functions of TFox.
 *This will dispath some custom events through the api.
 *Custom events are "setStatus"
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

function log(str){
    dump("[TFox] "+str + "\n");
}

var api = {};
var app;
/*
 * Strbundle
 */
var strbundle = null;
function initBundle(){
    var strb = Cc["@mozilla.org/intl/stringbundle;1"]
                .getService(Ci.nsIStringBundleService)
                .createBundle("chrome://tuenti/locale/common.properties");
    strbundle = {
        getString: function(key){
            try{
                return strb.GetStringFromName(key);
            }catch(e){
                return "Error: key not found";
            }
        },
        getFormattedString: function(key,vals){
            return strb.formatStringFromName(key, vals, vals.length);
        }
    };
    
    app.storage.set("TuentiStrbundle",strbundle);
}


/*
 *Refresh timer
 */
var timer = null;
function initTimer(){
    if(timer !== null)return;
    timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    app.storage.set("TuentiTimerStart",startTimer);
    app.storage.set("TuentiTimerStop",stopTimer);
}
function startTimer(){
    if(!timer || !api.TuentiProxy.isConnected())return;
    var prefs = Cc["@mozilla.org/preferences-service;1"]
          .getService(Ci.nsIPrefBranch);
    var interval = prefs.getIntPref("extensions.tuentifox.datos.refreshTime");
    
    var event = {
       notify: function(timer) {
            api.TuentiProxy.refresh();  
       }
    };
    timer.initWithCallback(event,interval,Ci.nsITimer.TYPE_REPEATING_SLACK);
}
function stopTimer(){
    if(timer)timer.cancel();
}
/*
 *Alerts
 */
function showAlerts(){
    var alertsService = Cc["@mozilla.org/alerts-service;1"]
              .getService(Ci.nsIAlertsService);
    var listener = {
        observe: function(subject, topic, data) {
            if (topic == "alertclickcallback") {
                var ios = Cc["@mozilla.org/network/io-service;1"]
                            .getService(Ci.nsIIOService);
                var uri = ios.newURI(data, null, null);
                var newTab = app.activeWindow.open(uri);
                newTab.focus();
                new Scheduler(api.TuentiProxy.refreshAlerts, 5000);
            }else if(topic == "alertfinished"){
                new Scheduler(showAlerts, 5000);
            }
        }
    }
    for (var i = 0; i < api.TuentiProxy.updates.length; i++) {
        if (!api.TuentiProxy.updates[i].displayed) {
            var text = api.TuentiProxy.updates[i].txt;
            var link = api.TuentiProxy.updates[i].url;
            api.TuentiProxy.updates[i].displayed = true;
            alertsService.showAlertNotification(
                "http://m.tuenti.com/favicon.ico",
                strbundle.getString("newupdate"),
                text, true, link, listener);
            return;
        }
    }
}

/*
 *Conect function
 */
var updated = false;
var endEventId = null;
function tuentiConnect(user,pass,create,logout,notopen){
    api.TuentiProxy.disconect();
    if(api.TuentiProxy.isConnected())return;
    
    api.TuentiProxy.dispatchEvent("navigateSidebar",
        {url:'chrome://tuenti/content/tuentiUserPass.xul',open:false});
    
    if(create||logout){
        if(create){
            api.TuentiProxy.dispatchEvent("navigateSidebar",
                {url:'chrome://tuenti/content/tuentiUserPass.xul',open:true});
        }
        return;
    }
    
    if(!user || user=="" || api.TuentiProxy.isConnected())return;
    pass = (pass && pass!="")?pass:getpass(user);

    api.TuentiProxy.dispatchEvent("navigateSidebar",
        {url:'chrome://tuenti/content/tuentiLoading.xul',open:!notopen});
    
    if(!updated){
        updated = true;
        var args = arguments;
        loadPatterns(function(){
            tuentiConnect.apply(this,args);
        });
        return;
    }
    
    api.TuentiProxy.login(user, pass);
    if(endEventId)api.TuentiProxy.removeEventListener(endEventId);
    endEventId = api.TuentiProxy.addEventListener("connect",function(){
        endEventId = null;
        api.TuentiProxy.dispatchEvent("navigateSidebar",
            {url:'chrome://tuenti/content/tuentiWindow.xul',open:!notopen});
        return 0;
    });
}
function getpass(username){// Get the password for the defined username
    //si no se ha especificado usuario, se usa el predeterminado
    if(!username)username = app.prefs.getValue("extensions.tuentifox.datos.usuario", "");
    var password;
    
    if (!username || username == "") {
        password = "";
    } else {
        var hostname = 'chrome://tuenti';
        var formSubmitURL = 'User - Pass';
        var httprealm = null;
        try {
           var myLoginManager = Cc["@mozilla.org/login-manager;1"]
                                  .getService(Ci.nsILoginManager);
           var logins = myLoginManager.findLogins({}, hostname, formSubmitURL, httprealm);
           for (var i = 0; i < logins.length; i++) {
              if (logins[i].username == username) {
                 password = logins[i].password;
                 break;
              }
           }
        }
        catch(ex) {}
    }
    return password;
}

/*
 *Global events
 */
function initGlobalEvents(){
    api.TuentiProxy.addEventListener("error",showError);
    api.TuentiProxy.addEventListener("notifications",showAlerts);
    api.TuentiProxy.addEventListener("refreshEnd",restartLastAct);
}
function showError(e){
      if(e.critical){
          api.TuentiProxy.disconect(true);
          var alertsService = Cc["@mozilla.org/alerts-service;1"].getService(Ci.nsIAlertsService);
          alertsService.showAlertNotification("http://tuenti.com/favicon.ico", strbundle.getString("erroralerttitle"), getErrorString(e.errorcde));
          api.TuentiProxy.dispatchEvent("setStatus",{status:4}); //Set status to "Error"
      }else{
          app.console.log(getErrorString(e.errorcde));
      }
}
function getErrorString(error){
    try{
      var errord = strbundle.getString("error"+error);        //get error description
    }catch(e){
    var errord = strbundle.getString("unknownerror"); //get unknown error description
    }
    return strbundle.getFormattedString("error",[errord,error]);
}


/*
 *Update 
 */
function url(spec) {
  var ios = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
  return ios.newURI(spec, null, null);
}
function alert(s){
    app.console.log(s);
    log(s);
    Cc["@mozilla.org/alerts-service;1"]
        .getService(Ci.nsIAlertsService)
        .showAlertNotification("http://tuenti.com/favicon.ico","error",s);
}

function createXHR(){
    return Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
             .createInstance(Ci.nsIJSXMLHttpRequest);
}

function showAlert(msg){
    var alertsService = Cc["@mozilla.org/alerts-service;1"]
            .getService(Ci.nsIAlertsService);
    alertsService.showAlertNotification("http://tuenti.com/favicon.ico","error",s);
}

const updateURL = 'http://www.mozilla-hispano.org/archivos/labs/tfox/update/update-%%VER%%.txt?rnd=' + (new Date()).getTime();

function isUpdateTime(){
    var enable = app.prefs.getValue('extensions.tuentifox.datos.enableupdate',true);
    if(!enable)return false;
    
    var date = app.prefs.getValue('extensions.tuentifox.datos.lastupdatedate',0);
    var delta = getTimestamp() - date;
    var updateDays = app.prefs.getValue('extensions.tuentifox.datos.updatedays',1);
    
    return delta > (updateDays*24*60*60);
}

function getRemoteVersion(callback){
    try{
        //FIXME: app.extensions undefined en ffx nighties: algun cambio de API?
        var tfxver = app.extensions.get('tuentifox@mozilla-hispano.org')
                .version.match(/^([0-9\.]+)/)[1];
        var url = updateURL.split("%%VER%%").join(tfxver);
        var xhr = createXHR();
        xhr.mozBackgroundRequest = true;
        xhr.open("GET",url,true);
        xhr.overrideMimeType('text/plain');
        xhr.onreadystatechange = function(){
            if(xhr.readyState == 4){
                if(xhr.status != 200){
                    //something went wrong
                    log("ERROR: Update server responsed status code "+xhr.status)
                    callback(0,"");
                }else{
                    var res = xhr.responseText.replace(/\s/g,"").split(',');
                    if(res[1] !=="false"){
                        showAlert(strbundle.getString("notsupported"));
                        log("Not supported TFox version")
                        callback(0,"");
                    }else{
                        log("Server version " + res[0] + ", url: " + res[3])
                        callback(res[0],res[3] + "?rnd=" + (new Date().getTime()));
                    }
                }
            }
        }
        xhr.send(null);
    }catch(e){
        log("XHR error " +e );
        callback(0,"");
    }
}

function getLocalVersion(){
    var tree = getPattersTree();
    if(tree){
        return tree.getElementsByTagName("TFox")[0].getAttribute("patternsRevision");
    }
    return -1;
}

function toNsURI(aPath){
    return Cc['@mozilla.org/network/io-service;1']
                .getService(Ci["nsIIOService"]).newURI(aPath, "UTF-8", null);
}

function saveURL(callback,nsiURL,nsiFile){
    var persist = Cc["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
            .createInstance(Ci.nsIWebBrowserPersist);

    persist.progressListener = {
        onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
            if((aStateFlags & 0x000000000010) == 0x10){
                callback();
            }
        },
        onProgressChange: function(){}
    };
    persist.saveURI(nsiURL, null, null, null, "", nsiFile);
}

function downloadUpdate(callback,url){
    log("downloading update from " + url);
    var chrome_url = 'chrome://tuenti/content/tuentiPatters.xml';
    var rv = Cc['@mozilla.org/chrome/chrome-registry;1']
                .getService(Ci["nsIChromeRegistry"]).convertChromeURL(toNsURI(chrome_url)).spec;
    
    var file_url = (/^file:/.test(rv)?"":"file://") + rv ;
    
    var file = Cc["@mozilla.org/network/protocol;1?name=file"]
        .createInstance(Ci.nsIFileProtocolHandler)
        .getFileFromURLSpec(file_url);

    var wbp = Cc['@mozilla.org/embedding/browser/nsWebBrowserPersist;1']
        .createInstance(Ci.nsIWebBrowserPersist);
    
    var uri = toNsURI(url);
    saveURL(callback,uri,file);
}

function getTimestamp(){
    return Math.round(new Date().getTime()/1000);
}

function getPattersTree(){
    try{
        var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
                        .createInstance(Ci.nsIJSXMLHttpRequest);
        xhr.open('GET','chrome://tuenti/content/tuentiPatters.xml',false);
        xhr.send(null);
        return xhr.responseXML;
    }catch(e){
        log("ERROR reading patterns tree");
        return false;
    }
}

function updatePatterns(callback){
    if(isUpdateTime()){
        log("Update time");
        new Scheduler(function(){
            getRemoteVersion(function(serverVersion,url){
                //Nota: esto va a fallar a partir del 19 de enero de 2038 a las 03:14:07 UTC
                app.prefs.setValue('extensions.tuentifox.datos.lastupdatedate',getTimestamp());
                var localVersion = getLocalVersion();
                log("Local Version: " + localVersion);
                if(serverVersion > localVersion){
                    downloadUpdate(callback,url);
                }else{
                    log("Client pattern revision is OK");
                    callback();
                }
            });
        },10);
        return;
    }else{
        log("Not update time");
        callback();
    }
}

function loadPatterns(callback){
    log("loadPatterns");
    updatePatterns(function(){
        var tree = getPattersTree();
        parsePatterns(tree);
        callback();
    });
}

function parsePatterns(tree){
    //*ifdebug*/dump("parseando datos...\n");
    var regexps = tree.getElementsByTagName("regexps")[0];
    var strings = tree.getElementsByTagName("strings")[0];
    var misc = tree.getElementsByTagName("misc")[0];
    
    var rxps = {};
    for each(var k in regexps.childNodes){
        if(k.nodeType!=1)continue;
        rxps[k.getAttribute('name')] = new RegExp(k.firstChild.nodeValue,k.getAttribute('flags'));
    }
    
    var strs = {};
    for each(var k in strings.childNodes){
        if(k.nodeType!=1)continue;
        strs[k.getAttribute('name')] = k.firstChild.nodeValue;
    }
    
    var _misc = {};
    for each(var k in misc.childNodes){
        if(k.nodeType!=1)continue;
        var name = k.getAttribute('name');
        switch(k.tagName.toLowerCase()){
            case 'bool':
                _misc[name] = k.firstChild.nodeValue == 'true';
                break;
            case 'int':
                _misc[name] = parseInt(k.firstChild.nodeValue);
                break;
            case 'str':
                _misc[name] = k.firstChild.nodeValue;
                break;
        }
    }
    api.TuentiProxy.setParseData(rxps,strs,_misc);
    //*ifdebug*/dump("Proceso de actualizacion terminado");
}

/*
 *Update time
 */
var lastActTimestamp = 0;
var lastActTimer = null;
function refreshLastUpdate(){
      var now = new Date().getTime();
      if(lastActTimestamp==0 || !api.TuentiProxy.isConnected())return;
      var ms = now-lastActTimestamp;
      var min = Math.round(ms/60000);
      //*ifdebug*/dump(min);
      api.TuentiProxy.dispatchEvent("lastActChanged",{min:min});
}
function restartLastAct(){
    lastActTimestamp =  (new Date()).getTime();
}
function initLastAct(){
    lastActTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    lastActTimer.initWithCallback({notify: refreshLastUpdate},60000,Ci.nsITimer.TYPE_REPEATING_SLACK);
}


/*
 *Private browsing watcher
 */
var restorePModeActions = [];
function watchForPrivateMode(){
    var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
    var obs = {observe:function(aSubject, aTopic, aData) {
    try{
        if (aTopic == "private-browsing") {
          if (aData == "enter"){
            //*ifdebug*/dump("enter\n");
            api.TuentiProxy.clearEvents();
            api.TuentiProxy.disconect(true);
          }else if (aData == "exit"){
            //*ifdebug*/dump("exit\n");
            api.TuentiProxy.disconnect(true);
            initGlobalEvents();
          }
        } else if (aTopic == "quit-application") {
          os.removeObserver(obs, "quit-application");
          os.removeObserver(obs, "private-browsing");
        }
    }catch(e){
        //*ifdebug*/dump("Error 1: " + e + "\n");
    }
    }};
    os.addObserver(obs, "private-browsing", false);
    os.addObserver(obs, "quit-application", false);
}

/*
 *Init
 */
var inited = false;
function initGlobal(){
    if(inited)return;
    inited = true;
    app = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);
    Components.utils.import("resource://tuenti/tuentiAPI.jsm",api);
    initBundle();
    initTimer();
    initLastAct();
    initGlobalEvents();
    watchForPrivateMode();
    app.storage.set("TuentiReconectFnc",tuentiConnect);
    if(app.prefs.getValue("extensions.tuentifox.datos.autoconect",false)) //if "autoconnect" pref is true
    tuentiConnect(app.prefs.getValue("extensions.tuentifox.datos.usuario",""),null,false,false,true); //Conect to the default user
}



/*
 *Utils
 */

function openDialog(url,title,parms){
    var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
    var win = ww.openWindow(null,url,title,parms, null);

}
function Scheduler(callback, callAfter_MS){
    this.callback_ = callback;
    this.timer_ = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);// get a timer
    var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);// we want to stop the beat on shutdown
    observerService.addObserver(this, "xpcom-shutdown", false);
    this.timer_.initWithCallback(this, callAfter_MS, this.timer_.TYPE_ONE_SHOT);// initialize the timer to fire after the given ms interval
}
Scheduler.prototype.QueryInterface = function(iid){
    if (   iid.equals(Ci.nsISupports)
        || iid.equals(Ci.nsITimerCallback)
        || iid.equals(Ci.nsIObserver))
        return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
}
Scheduler.prototype.observe = function(aSubject, aTopic, aData){
    if (aTopic == "xpcom-shutdown")// stop the beat on shutdown (see http://wiki.mozilla.org/XPCOM_Shutdown)
        this.stop();
}
Scheduler.prototype.stop = function(){
    if (this.timer_) {
        this.timer_.cancel();// stop the timebeat and remove the observer
        this.timer_ = null;
        this.callback_ = null;
        var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        observerService.removeObserver(this, "xpcom-shutdown");
    }
}
Scheduler.prototype.notify = function(aTimer){
    this.callback_();
    this.stop();
}



/*
 *Groups
 */
var TuentiGroups = {
    _open: false,
    _con: null,
    open: function(){
        if(TuentiGroups._open)return false;
        TuentiGroups._open = true;
        var file = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
        file.append("TFox.sqlite");
        if(!file.exists()){
            file.create(Ci.nsIFile.NORMAL_FILE_TYPE,438);
        }
        var storageService = Cc["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);
        TuentiGroups._con = storageService.openDatabase(file);
        if(TuentiGroups._con.connectionReady){
            TuentiGroups.check();
            return true;
        }
        return false;
    },
    close:function(){
        if(TuentiGroups._con)TuentiGroups._con.close();
        TuentiGroups._open = false;
    },
    check: function(){
        var c = TuentiGroups._con;
        if(!c.tableExists("grupos")){
            c.createTable("grupos",'"name" VARCHAR NOT NULL ,"id" INTEGER NOT NULL, "fav" INTEGER NOT NULL DEFAULT 0');
        }
        if(!c.tableExists("friends")){
            c.createTable("friends",'"id" VARCHAR NOT NULL , "group" INTEGER NOT NULL DEFAULT 0');
        }
    },
    getGroupsAndPeople: function(){
        var g = TuentiGroups.getGroups();
        var c = TuentiGroups._con;
        var ret = [];
        for each(var k in g){
            var s = c.createStatement("SELECT \"id\" FROM 'friends' WHERE \"group\"='"+k[2]+"'");
            var f = [];
            while(s.executeStep())
                f.push(s.getInt32(0));
            ret.push({name:k[0],fav:k[1],id:k[2],friends:f});
        }
        return ret;
        
    },
    addGroup: function(fid,group){
    var c = TuentiGroups._con;
        var s = c.createStatement("SELECT \"id\" FROM 'grupos' WHERE name='"+group+"'");
        if(s.executeStep()){
            var gid = s.getInt32(0);
        }else{
            var gid = newGroup(group);
        }
        if(gid <= 0)return;
        var s2 = c.createStatement("INSERT INTO 'friends' ('id','group') VALUES ('"+fid+"','"+gid+"')");
        s2.execute();
    },
    newGroup: function(name,fav){
        var f = fav?fav:false;
        var c = TuentiGroups._con;
        var s = c.createStatement("SELECT MAX(id) FROM 'grupos'");
        if(s.executeStep()){
            var id = s.getInt32(0);
            if(!id)id=0;
            id++;
            var s2 = c.createStatement("INSERT INTO 'grupos' ('name','id','fav') VALUES ('"+name+"','"+id+"','"+(f?1:0)+"')");
            s2.execute();
            return id;
        }else{
            throw "SQLlite error";
        }
    },
    removeGroup: function(id,group){
        var c = TuentiGroups._con;
        var s = c.createStatement("SELECT id FROM 'grupos' WHERE name='"+group+"'");
    if(!s.executeStep())return;
        var gid = s.getInt32(0);
    if(gid <=0){
        return;
    }
        var s2 = c.createStatement("DELETE FROM 'friends' WHERE id='"+id+"' and \"group\"='"+gid+"'");
        s2.execute();
    },
    getGroups: function(id){
        var c = TuentiGroups._con;
        var ret = [];
        if(id)
            var s = c.createStatement("SELECT name,fav,id FROM 'grupos' WHERE id='"+id+"'");
        else
            var s = c.createStatement("SELECT name,fav,id FROM 'grupos'");
        while(s.executeStep()){
            ret.push([s.getString(0),s.getInt32(1)!=0,s.getInt32(2)]);
        }
        return ret;
    },
    getGroupsOfUser: function(id){
    if(!id)return false;
        var c = TuentiGroups._con;
        var ret = [];
    var s = c.createStatement("SELECT \"group\" FROM 'friends' WHERE id='"+id+"'");
        while(s.executeStep()){
            var g = TuentiGroups.getGroups(s.getInt32(0));
        if(g[0])ret.push(g[0]);
        }
    return ret;
    },
    setGroupFav: function(group,fav){
        var c = TuentiGroups._con;
        var s2 = c.createStatement("UPDATE \"grupos\" SET fav='"+(fav?1:0)+"' WHERE name='"+group+"'");
        s2.execute();
    }
    
};



var TuentiTheme = {
    getCurrentTheme : function(){
    return app.prefs.getValue("extensions.tuentifox.datos.web.theme","none");
    },
    setCurrentTheme: function(newtheme,test,nocache){
    if(!test){
        app.prefs.setValue("extensions.tuentifox.datos.web.theme",newtheme);
    }
    //Ahora vamos a recorrer todas las paginas abiertas, cambiando el theme
    var wm = Cc["@mozilla.org/appshell/window-mediator;1"]  
               .getService(Ci.nsIWindowMediator);  
    var enumerator = wm.getEnumerator("navigator:browser");  
    while(enumerator.hasMoreElements()){
        var win = enumerator.getNext();
        if(win.gBrowser){
        var browsers = win.gBrowser.browsers;
        for(var i = 0, l=browsers.length; i<l; i++){
            var browser = browsers[i];
            try{
            if(browser.currentURI.host == "www.tuenti.com"){
                var elm = browser.contentDocument.getElementById("TFoxTheme");
                if(elm){
                elm.setAttribute("href",
                    'chrome://tuentiaccessible/content/themes/'+newtheme+'/theme.css?' +(nocache?new Date().getTime():"") );
                }
            }
            }catch(e){}
        }
        }
    }  
    }
}


//Uncoment this for developing themes. Will update the theme automatically
/*function ti(){
    try{
    TuentiTheme.setCurrentTheme(TuentiTheme.getCurrentTheme(),true,true);
    }catch(e){dump(e);}
    new Scheduler(ti,500);
}

new Scheduler(ti,100);
//*/
