/* 
    This file is part of TFox.

    TFox is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TFox. If not, see <http://www.gnu.org/licenses/>.
*/

var EXPORTED_SYMBOLS = ["TuentiProxy"]

/**
 * Tuenti API - API that connects Tuenti (spanish social web)
 *
 * Tuenti have not public api, so this works using the normal web request and parsing the responses
 *
 * Login process:
 * -Get home-page
 * -Crypt the pass with the timestamp in the homepage and send it to the server
 * -Check that the password is ok.
 * -Get profile home-page and extract the csfr (a kind of session id)
 * -Get the others pages with the csfr and parse it
 *
 * @author David Bengoa (YouWoTMA) youwotma@hotmail.es
 * @version 1.0
 * @class TuentiApi
 */


/**
 * METHODS :
 * isConnected()    
 * login (user, mdp, listener)  
 * getStatus()
 * setStatus(msg)	
 * sendMessage(who, what)        			//alias of   sendPrivateMessage(who,wath)  
 * sendPrivateMessage(who,wath)
 * sendTablonMessage(who,wath)
 * flushUpdates()
 *
 *
 * PROPERTIES :
 * Note: some of this properties have changed
 * 
 * updates [{txt, url, displayed}]  
 * friends [{user_id, nick, status, imgUrl,url}]      //now, act and change objects in news are too en friends
 * news[{
 *	user_id,
 *	nick,
 *	imgUrl, 
 *	url, 
 *	change,
 *	status, 
 *	act: {	
 *		 blog:{url,title,text},
 *		picts:{url,count},
 *		friends:{url,count,name},
 *		comments:{url,count,from},
 *		thumbs:[{url,src}]
 *	}]
 *	
 *       inbox   [{id, imgUrl,imgf,nick,when,txt,nuevo,leeido,readUrl}]
 *	imagestag  [{url,imgurl,imgf,title,uploader,upid}]
 *	comments [{user_id,imgurl,imgf,nick,responseinfo,responsed,deleteinfo,when,text}]
 *
 */
const TUENTI_HOST       = "http://www.tuenti.com";
const MIN_IMAGE_SIZE    = 1;                      //TODO: discover this and
const MAX_IMAGE_SIZE    = 99999999999999;         // this
const CONECTION_TIMEOUT = 10000;                  //Request timeout (will retry if reached)         //10 seconds
const htmlEntities      = {nbsp:160,iexcl:161,cent:162,pound:163,curren:164,yen:165,brvbar:166,sect:167,uml:168,copy:169,ordf:170,laquo:171,not:172,shy:173,reg:174,macr:175,deg:176,plusmn:177,sup2:178,sup3:179,acute:180,micro:181,para:182,middot:183,cedil:184,sup1:185,ordm:186,raquo:187,frac14:188,frac12:189,frac34:190,iquest:191,Agrave:192,Aacute:193,Acirc:194,Atilde:195,Auml:196,Aring:197,AElig:198,Ccedil:199,Egrave:200,Eacute:201,Ecirc:202,Euml:203,Igrave:204,Iacute:205,Icirc:206,Iuml:207,ETH:208,Ntilde:209,Ograve:210,Oacute:211,Ocirc:212,Otilde:213,Ouml:214,times:215,Oslash:216,Ugrave:217,Uacute:218,Ucirc:219,Uuml:220,Yacute:221,THORN:222,szlig:223,agrave:224,aacute:225,acirc:226,atilde:227,auml:228,aring:229,aelig:230,ccedil:231,egrave:232,eacute:233,ecirc:234,euml:235,igrave:236,iacute:237,icirc:238,iuml:239,eth:240,ntilde:241,ograve:242,oacute:243,ocirc:244,otilde:245,ouml:246,divide:247,oslash:248,ugrave:249,uacute:250,ucirc:251,uuml:252,yacute:253,thorn:254,yuml:255,quot:34,amp:38,lt:60,gt:62,apos:39,OElig:338,oelig:339,Scaron:352,scaron:353,Yuml:376,circ:710,tilde:732,ensp:8194,emsp:8195,thinsp:8201,zwnj:8204,zwj:8205,lrm:8206,rlm:8207,ndash:8211,mdash:8212,lsquo:8216,rsquo:8217,sbquo:8218,ldquo:8220,rdquo:8221,bdquo:8222,dagger:8224,Dagger:8225,permil:8240,lsaquo:8249,rsaquo:8250,euro:8364,fnof:402,Alpha:913,Beta:914,Gamma:915,Delta:916,Epsilon:917,Zeta:918,Eta:919,Theta:920,Iota:921,Kappa:922,Lambda:923,Mu:924,Nu:925,Xi:926,Omicron:927,Pi:928,Rho:929,Sigma:931,Tau:932,Upsilon:933,Phi:934,Chi:935,Psi:936,Omega:937,alpha:945,beta:946,gamma:947,delta:948,epsilon:949,zeta:950,eta:951,theta:952,iota:953,kappa:954,lambda:955,mu:956,nu:957,xi:958,omicron:959,pi:960,rho:961,sigmaf:962,sigma:963,tau:964,upsilon:965,phi:966,chi:967,psi:968,omega:969,thetasym:977,upsih:978,piv:982,bull:8226,hellip:8230,prime:8242,Prime:8243,oline:8254,frasl:8260,weierp:8472,image:8465,real:8476,trade:8482,alefsym:8501,larr:8592,uarr:8593,rarr:8594,darr:8595,harr:8596,crarr:8629,lArr:8656,uArr:8657,rArr:8658,dArr:8659,hArr:8660,forall:8704,part:8706,exist:8707,empty:8709,nabla:8711,isin:8712,notin:8713,ni:8715,prod:8719,sum:8721,minus:8722,lowast:8727,radic:8730,prop:8733,infin:8734,ang:8736,and:8743,or:8744,cap:8745,cup:8746,int:8747,there4:8756,sim:8764,cong:8773,asymp:8776,ne:8800,equiv:8801,le:8804,ge:8805,sub:8834,sup:8835,nsub:8836,sube:8838,supe:8839,oplus:8853,otimes:8855,perp:8869,sdot:8901,lceil:8968,rceil:8969,lfloor:8970,rfloor:8971,lang:9001,rang:9002,loz:9674,spades:9824,clubs:9827,hearts:9829,diams:9830};   //html table
const MAX_RETRIES       = 4;                      //Max number of retry's. a retry may be by a timeout or by a server error This is for only one page. The API will retry failed request a maxium of MAX_RETRIES times
const MAS_LOGIN_RETRIES = 4;                      //Max numver of retry's in login.
var sBox                =  new Components.utils.Sandbox("http://www.tuenti.com/");
const TuentiStatus      = {disconected:0,conecting:1,conected:2,refresh:3,relogin:4};  //Tuenti status The connection status to tuenti is stablished by a int The values can be: 0-disconected 1-conecting 2-conected 3-conected+refresh 4-conected+relogin+refresh

/*var REGEXPS = {};//RegExps
"HOME_TIMESTAMP,HOME_TIMESTAMPS,ACT_FOTOS,ACT_FOTOS_UP,ACT_COMEN,ACT_FRIEND,ACT_PICTURES,ACT_BLOG,ACT_PHOTO,ACT_NAME,ACT_CAMBIOS,NOTIFICACIONES,FRIEND_ID,FRIEND_IMG,FRIEND_STATUS,INBOX_ID,INBOX_IMG,INBOX_WHEN,INBOX_TEXT,INBOX_FROM_PHONE,IMG_SIZE,IMG_PERFIL,COM_USER,COM_RESPONSE,COM_DELETE,COM_WHEN,YOUR_INFO,ALL_PHOTO".split(",").forEach(function(k){  REGEXPS[k] = / /; });
const KEEP_VARIABLES       = " uploadCluster global_uid csfrChallenge SUBDOMAIN "; //variables to keep from the home of tuenti
var STRINGS = {}; //Strings
"ACTS_START_TAG,ACT_START_TAG,ACTS_END_TAG,ACT_SPLIT_A,ACT_SPLIT_B,ESTOY_START,DIV_END,NOTIFICACIONES_START,FRIENDS_SPLIT,FRIENDS_END,CSFR_START,LOADING,MSGS_SPLIT,MSGS_END,READ_URL_START,READ_URL_END,IMG_UPLOAD_ID_START,PROFILE_IMAGES_SPLIT,PROFILE_IMAGES_END,PROFILE_COM_SPLIT,PROFILE_COM_END,PROFILE_COM_TXT_START,PROFILE_COM_TXT_END,PROFILE_COM_NEW,TUENTI_MANTENIMIENTO,ESTADO_CAMBIADO,MENSAGE_OK,FOTO_PPAL_START,ALL_PHOTOS_START,ALL_FRIENDS_START,CSFR_ST,CSFR_EN".split(",").forEach(function(k){  STRINGS[k] = ""; });
var MISC = {};*/

var REGEXPS = {};

"IMG_ST,ACT_ESTADO,ACT_THUMB,ACT_FOTOS,ACT_FOTOS_TAG,ACT_COMEN,ACT_FRIEND,ACT_BLOG,ACT_COMEN_PREV_OTHER,ACT_COMEN_PREV,GREEN,IMG_SIZE,MSG,IMG_PERFIL,COM_PERFIL,CSFR,STATUS_CHANGE_STATUS,EVENT,MSG_FULL_URL,EMO"
.split(",").forEach(function(i){
    REGEXPS[i] = / /;
});

var STRINGS={};
"CONFIG_ST,FDATA_ST,FDATA_END,ROUTER_ST,ROUTER_END,ROUTER_ADD,CONFIG_EN,ESTADO_ST,ESTADO_EN,NEWS_ST,NEWS_EN,NEWS_SPLIT,GREEN_ST,GREEN_EN,IMG_EN,ACTS_START,ACT_SPLIT,ACT_LINK_ST,ACT_LINK_EN,MSG_ST,MSG_EN,MSG_SPLIT,MSG_NEW,PROFILE_IMAGES_SPLIT,PROFILE_IMAGES_END,COM_SPLIT,COM_END,COM_TXT_ST,COM_TXT_EN,COM_REPLACE_FIND,COM_REPLACE_REPLACE,DISCONECTED_MSG,STATUS_CHANGE_SUCCESS,MSG_SEND_SUCCESS,COM_SEND_SUCCESS,PHOTO_UPLOAD_URL,PHOTO_UPLOAD_REF,PHOTO_ID_ST,PHOTO_ID_EN,PHOTO_CONF_SEND,EVENT_ST,EVENT_EN,EVENT_HOY,EVENT_MANANA,EVENT_SEMANA,EVENT_PROX,ALLFRIENDS_ST,ALLFRIENDS_EN,FMSG_ST,FMSG_EN,CHAT_INFO_EN,CHAT_INFO_ST,ID_IS_OK,CSFR_ST,CSFR_EN,PROFILE_IMAGES_ST"
.split(",").forEach(function(i){
    STRINGS[i] = " ";
});


var MISC = {};
"urlformat,home_host,home_url,domain,computeurl,setstatusurl,sendmsgurl,sendwallurl,loginurl,homefragment,homemigas,homeajax,inboxurl,profileurl,allfriendsurl,sendmsgparms,sendwallparms,loginparms,statusparms,preprocess,nophotourl"
.split(",").forEach(function(i){
    MISC[i] = " ";
});
MISC.allfriendsinhome = true;


var externPreprocessfn = function(a,b){return b;};
function setupPreprocess(){
    if(!MISC.preprocess)return;
    
    var fn = Components.utils.evalInSandbox(MISC.preprocess,sBox);
    if(fn && typeof fn == 'function'){
	externPreprocessfn = fn;
    }
}

function preprocess(mode,data){
    if(!data)return null;
    var pdata = externPreprocessfn(mode,data);
    return (typeof data === typeof pdata)?pdata:data;
}

/****************** 
 *  TASK MANAGER  * 
 ******************/
const SYNC_TASK  = 1;
const ASYNC_TASK = 2;
var taskManager = {
    _tasks : [],
    rev: 1,    
    lastTaskTimestamp: +new Date(),
    asyncTaskRuning: false,
    addTask: function(fn,type){
        taskManager._tasks.push([fn,type]);
        taskManager._tasksChanged();
    },
    _tasksChanged: function(){
        if(taskManager.asyncTaskRuning)return;
        var run = taskManager._tasks.shift();
        if(!run){
            TuentiProxy.dispatchEvent("refreshEnd",{});
            return;  //No more work
        }
        TuentiProxy.dispatchEvent("refreshStart",{});
	
	taskManager.lastTaskTimestamp = +new Date();
        
	if(run[1] === SYNC_TASK){
            run[0]();
            taskManager._tasksChanged();
            return;
        }else if(run[1] == ASYNC_TASK){
            taskManager.asyncTaskRuning = true;
            run[0]((function(){
		var mirev = taskManager.rev;
		return function(){
		    if(mirev !== taskManager.rev)return;
		    //*ifdebug*/log("async task end");
		    taskManager.asyncTaskRuning = false;
		    taskManager._tasksChanged();
		};
	    })());
            return;
        }
        
    },
    isBussy: function(){
	
	
	var now = +new Date();
	var time = now - taskManager.lastTaskTimestamp;
	if(time > 1000*60*2){
	    taskManager._tasks = [];
	    taskManager.asyncTaskRuning = false;
	    taskManager.rev++;
	    if(handler.status > TuentiStatus.conected)handler.status = TuentiStatus.conected;
	    return false;
	}
	
	if(taskManager._tasks.length === 0 && !taskManager.asyncTaskRuning)return false;
        else return true;
    },
    haveAKitKat: function(){
        taskManager._tasks = [];
        taskManager.asyncTaskRuning = false;
	taskManager.rev++;
    }
};

//Al parecer nsIJSON funciona en firefox 3.1 y 3.0.x

function JSONencode(object){
    return Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON).encode(object);
}

function JSONdecode(str){
    return Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON).decode(str);
}

/***********
 * HANDLER *
 ***********/
var refindex = 0;
var handler = {
    status: TuentiStatus.disconected,
    _con: null,
    _setStatus: function(sta){
        handler.status = sta;
        TuentiProxy.dispatchEvent("statusChange",{});
    },
    refresh: function(){
	if(taskManager.isBussy() || handler.status != TuentiStatus.conected)return;
        taskManager.addTask(TuentiProxy._parseHome2,ASYNC_TASK);
        taskManager.addTask(TuentiProxy._parseInbox,ASYNC_TASK);
        taskManager.addTask(TuentiProxy._parseProfile,ASYNC_TASK);
        taskManager.addTask(TuentiProxy.clean,SYNC_TASK);
        taskManager.addTask(handler.saveData,SYNC_TASK);
    },
    login: function(){
        handler._setStatus(TuentiStatus.conecting);
        taskManager.addTask(TuentiProxy._login,ASYNC_TASK);
        taskManager.addTask(TuentiProxy._parseHome,ASYNC_TASK);
        taskManager.addTask(handler.getLocalData,ASYNC_TASK);
    },
    disconnect: function(){
        handler._setStatus(TuentiStatus.disconected);
        taskManager.haveAKitKat();
        calcellAllConectios();
        TuentiProxy.dispatchEvent("disconect",{});
    },
    getFile: function(file){
        var f = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
        f.append("TFox");
        f.append(TuentiProxy.myemail.replace(/[^a-z0-9@\-_\.]/gi,"_"));
        f.append(file);
        return f;
    },
    getLocalData: function(f){
        var local = handler.getFile("data.json");  
 
        var corrupto = true;
 
        if(local.exists){
            var data = handler.loadJSON(local);
            if(data && typeof data == 'object' && data.inbox && data.comments && data.images){
                corrupto = false;
                TuentiProxy.inbox = data.inbox;
                TuentiProxy.comments = data.comments;
                TuentiProxy.imagestag = data.images;
                TuentiProxy.clean();
            }
        }
        
        if(corrupto){
            taskManager.addTask(TuentiProxy._parseInbox,ASYNC_TASK);
            taskManager.addTask(TuentiProxy._parseProfile,ASYNC_TASK);
            taskManager.addTask(TuentiProxy.clean,SYNC_TASK);
            taskManager.addTask(handler.saveData,SYNC_TASK);
        }
        
        taskManager.addTask(function(){
            //*ifdebug*/log("Acabado");
            handler._setStatus(TuentiStatus.conected);
            TuentiProxy.dispatchEvent("connect",{});
        },SYNC_TASK);
        
        
        taskManager.addTask(handler.saveImg,SYNC_TASK);

        //Esto equivaldria a una actualizacion normal
        taskManager.addTask(TuentiProxy._parseHome2,ASYNC_TASK);
        taskManager.addTask(TuentiProxy._parseInbox,ASYNC_TASK);
        taskManager.addTask(TuentiProxy._parseProfile,ASYNC_TASK);
        taskManager.addTask(TuentiProxy.clean,SYNC_TASK);
        taskManager.addTask(handler.saveData,SYNC_TASK);
        
        f();    //End this async function. Will start the another tasks
    },
    saveData: function(){
        handler.save({
            inbox : TuentiProxy.inbox,
            comments : TuentiProxy.comments,
            images : TuentiProxy.imagestag
        },"data.json");
    },
    save: function(what,where){
        //*ifdebug*/log("Guardando " + where);
        try{
            var data = JSONencode(what);
            var file = handler.getFile(where);
            if(!file.exists())file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE,0664);
            var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
            foStream.init(file, -1, -1, 0); 
            foStream.write(data,data.length);
            foStream.close();
        }catch(e){
            //*ifdebug*/log(e);
            return;
        }
    },
    saveImg: function(){
        if(handler.status < TuentiStatus.conected)return;
        try{
            var file = handler.getFile("foto.jpg");
            var url = TuentiProxy.myimg.func(120);
	    var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Components.interfaces.nsIWebBrowserPersist);
	    var obj_URI = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newURI(url, null, null);
            persist.saveURI(obj_URI, null, null, null, "", file);
        }catch(e){}
    },
    loadJSON: function(file){
        try{
            if(typeof file == typeof " ")file = handler.getFile(name);
            if(!file || !file.exists())return false;
            var data = "";
            var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
            var sstream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
            fstream.init(file, -1, 0, 0);
            sstream.init(fstream); 
            var str = sstream.read(4096);
            while (str.length > 0) {
              data += str;
              str = sstream.read(4096);
            }
            sstream.close();
            fstream.close();
            return JSONdecode(data);
        }catch(e){
            return {};
        }
    }
}


/*************
 *    API    *
 *************/
var TuentiProxy = {
          /////////////////////////////////////////////////////////////////////
         //                        PUBLIC VARIABLES                         //
        /////////////////////////////////////////////////////////////////////
	updates:      [],
        findex:       0,
	friends:      {},
	inbox:        [],
	imagestag:    [],
	comments:     [],
        events:       [],
	config:       {},
	myname:       "",
	myemail:      "",
	myimg:        {},
	pass:         "",
        route:      null,
	
	
	taskManager: taskManager,
	handler: handler,
	debugid: Math.ceil(Math.random()*1000),
	initts: +new Date(),
        
        get subdominio(){
            if(TuentiProxy.router)return TuentiProxy.router.home_farm || TuentiProxy.router.failover_farm || "www";
            return "www";
        },
        
          /////////////////////////////////////////////////////////////////////
         //                        API FUNCTIONS                            //
        /////////////////////////////////////////////////////////////////////
	login: function(user,pass){
		if(handler.status != TuentiStatus.disconected)return;
		TuentiProxy.myemail = user;
		TuentiProxy.pass = (pass);
                handler.login();
	},
        get status(){
            return handler.status;
        },
	disconect: function(critical){
            if(handler.status <= TuentiStatus.conecting && !critical)return;
            //Clear the data
            TuentiProxy.updates  = [];
            TuentiProxy.friends  = {};
            TuentiProxy.inbox    = [];
            TuentiProxy.imagestag= [];
            TuentiProxy.comments = [];
            TuentiProxy.config   = {};
            TuentiProxy.myname   = "";
            TuentiProxy.myemail  = "";
            TuentiProxy.myimg    = {};
            TuentiProxy._estoy   = "";
            TuentiProxy._csfr    = "";
            TuentiProxy._retries = 0;
	    TuentiProxy.chatData = null;
	    TuentiProxy.events   = [];
            handler.disconnect();
	},
	setStatus: function(msg,endff){
            var endf = endff || function(){};
            if(!msg || !TuentiProxy.status >= TuentiStatus.conected)return endf(false);
            txhr({
                url: TuentiProxy.computeURL(MISC.setstatusurl),
                method: "POST",
                callback: function(xhr,response){
		    response = preprocess("setstatus",response);
                    if(response.indexOf(STRINGS.STATUS_CHANGE_SUCCESS))
                        if(REGEXPS.STATUS_CHANGE_STATUS.test(response)){
                            TuentiProxy._estoy = RegExp.$1;
                            return endf(true);
                        }
                    return endf(false);
                },
                error: function(){
                    endf(false);
                },
                headers: {"Content-type":"application/x-www-form-urlencoded"},
                plain: TuentiProxy.computeComplexFilter(MISC.statusparms,{status: encodeURIComponent(msg),uid: TuentiProxy.myuid,csrf: TuentiProxy.csfr})
            });
            return true;
	},
	sendMessage: function(who,wath,f){
		return TuentiProxy.sendPrivateMessage(who,wath,f);
	},
        computeComplexFilter: function(filter,remplazos){
            var str = filter;
            for(var k in remplazos){
                str=str.split("[" + k + "]").join(remplazos[k]);
            }
            return str;
            
        },
	sendPrivateMessage: function(who,wath,endff){
            var endf = endff || function(){};
            if(!who || !wath || !TuentiProxy.status >= TuentiStatus.conected)return endf(false);
            txhr({
                url: TuentiProxy.computeURL(MISC.sendmsgurl),
                method: "POST",
                callback: function(xhr,response){
		    response = preprocess("pm",response);
                    if(response.indexOf(STRINGS.MSG_SEND_SUCCESS)>0)
                        endf(true);
                    else
                        endf(false);
                },
                error: function(){endf(false);},
                headers: {"Content-type":"application/x-www-form-urlencoded"},
                plain: TuentiProxy.computeComplexFilter( MISC.sendmsgparms, {who: who,what: encodeURIComponent(wath),csrf: TuentiProxy.csfr})
            });
            return true;
	},
	readMsg: function(readUrl){},
	readComments: function(){},
        getNewMsgCount: function(){
            var i=0;
            for each(var k in TuentiProxy.inbox){
                i += k.nuevo?1:0;
            }
            return i;
        },
        getNewComCount: function(){
            var i=0;
            for each(var k in TuentiProxy.comments){
                i += k.nuevo?1:0;
            }
            return i;
        },
	sendTablonMessage: function(who,wath,endff){
            var endf = endff || function(){};
            if(!who || !wath || !TuentiProxy.status >= TuentiStatus.conected)return endf(false);
            txhr({
                url: TuentiProxy.computeURL(MISC.sendwallurl.split("[who]").join(who)),
                method: "POST",
                callback: function(xhr,response){
		    response = preprocess("coment",response);
                    if(response.indexOf(STRINGS.COM_SEND_SUCCESS)>0)
                        endf(true);
                    else
                        endf(false);
                },
                error: function(){endf(false);},
                headers: {"Content-type":"application/x-www-form-urlencoded"},
                plain: TuentiProxy.computeComplexFilter(MISC.sendwallparms,{what: encodeURIComponent(wath),csrf: TuentiProxy.csfr,who:who})
            });
            return true;
	},
	getStatus: function(){
		return TuentiProxy._estoy;
	},
	flushUpdates: function(){
		TuentiProxy.updates = [];
	},
	isConnected: function(){
		return handler.status >= TuentiStatus.conected;
	},
        isConecting:function(){
                return handler.status == TuentiStatus.conecting;
        },
	refresh: function(){
		handler.refresh();
	},
	refreshAlerts: function(){
		if(handler.status != TuentiStatus.conected)return;
		handler._setStatus(TuentiStatus.refresh);
		TuentiProxy._parseHome2(function(){
		    handler._setStatus(TuentiStatus.conected);
		});
	},
        setParseData: function(rxs,str,msk){
            function merge(a,b){
                for(var k in b){
                    if(a[k] !== undefined && (typeof b[k] == typeof a[k])){
                        b[k] = a[k];
                    }else{
                        TuentiProxy.dispatchEvent("error",{errorcde:600,critical:true});
                        throw "600";
                    }
                }
            }
            merge(rxs,REGEXPS);
            merge(str,STRINGS);
            merge(msk,MISC);
	    
	    setupPreprocess();
        },
          /////////////////////////////////////////////////////////////////////
         //                        LOGIN ENGINE                             //
        /////////////////////////////////////////////////////////////////////
        _loginr: 0,
	_login: function(endf){
            txhr({
                url:MISC.home_url,
                galleta: "ourl=http%3A//www.tuenti.com/",
                retry: false,
                error: function(){ if(++TuentiProxy._loginr < MAS_LOGIN_RETRIES)TuentiProxy._login(endf); },
                callback: function(xhr,response){
                    
                    //*ifdebug*/ log(response);
		    
		    response = preprocess("login",response);
                    
                    txhr({
                        url: MISC.loginurl,
                        method: "POST",
                        retry:false,
                        error: function(){ if(++TuentiProxy._loginr < MAS_LOGIN_RETRIES)TuentiProxy._login(endf); },
                        galleta: '',
                        callback: function(xhr,response){
			    response = preprocess("login2",response);
			    
			    if(response.indexOf(STRINGS.ID_IS_OK) > 0)
				endf();
			    else
				TuentiProxy.dispatchEvent("error",{errorcde:300,critical:true});
				
                        },
                        headers: {
                        "Content-type":"application/x-www-form-urlencoded",
                        "Referer":"http://www.tuenti.com/?m=login",
                        "Pragma":"",
                        "Cache-Control":""
                        },
                        plain: TuentiProxy.computeComplexFilter(MISC.loginparms,{email: encodeURIComponent(TuentiProxy.myemail),pass: encodeURIComponent(TuentiProxy.pass)}),
                        galleta: "ourl=http%3A//www.tuenti.com/",
                        noreffer : true
                        
                    });
                    
                }
            });
	},

          /////////////////////////////////////////////////////////////////////
         //                        EVENTS ENGINE                            //
        /////////////////////////////////////////////////////////////////////
        addEventListener: function(event,callback){
            //refreshStart refreshEnd error relogin connect disconect notifications
            TuentiProxy._attachedEvents[TuentiProxy._attachedEvents.length] = {e:event,c:callback};
            return TuentiProxy._attachedEvents.length-1;
        },
        removeEventListener:function(listenerId){
            if(TuentiProxy._attachedEvents[listenerId])delete TuentiProxy._attachedEvents[listenerId];
        },
        dispatchEvent: function(event,data){
            //*ifdebug*/if(event !== "log")log("Dispatch event: " + event);
            var e = TuentiProxy._attachedEvents;
            for(var k in e)
                if(e[k].e == event){
                    try{
                        var a = e[k].c(data);
                    }catch(e){
                        //*ifdebug*/(event==="log"?dump:log)("TFox: Error in event handler:",e,event);
                    }
                    if(a === false)
                        return;
                    else if(a === 0)
                        delete e[k];
                }
        },
        clearEvents: function(){
            TuentiProxy._attachedEvents = [];
        },
        _attachedEvents:[],
          /////////////////////////////////////////////////////////////////////
         //                        PRIVATE PROPERTIES                       //
        /////////////////////////////////////////////////////////////////////
	_estoy: "",
	_csfr: "",
          /////////////////////////////////////////////////////////////////////
         //                        PARSING FUNCTIONS                        //
        /////////////////////////////////////////////////////////////////////
        computeURL: function(parms){
            return MISC.computeurl.split("[subdominio]").join(TuentiProxy.subdominio).split("[dominio]").join(MISC.domain).split("[parms]").join(parms);
        },
	_getBou: function(){
		return '---------------------------' + (new Date().getTime());
	},
        clean: function(){
            TuentiProxy.checkImgf();
            TuentiProxy.reorderFriends();
            for (var k in TuentiProxy.friends)
                  if(!TuentiProxy.friends[k].imgf || !TuentiProxy.friends[k].imgURL)delete TuentiProxy.friends[k];
        },
        checkImgf: function(){
            for each(var k in TuentiProxy.friends){
                if(k.imgURL && !k.imgf)k.imgf = imgfun(k.imgURL);
                if(k.act && k.act.thumbs)
                    for each(var k in k.act.thumbs)
                        k.imgf = imgfun(k.src);
            }
            for each(var k in TuentiProxy.comments)
                if(k.imgURL && !k.imgf)k.imgf = imgfun(k.imgURL);
            for each(var k in TuentiProxy.inbox)
                if(k.imgURL && !k.imgf)k.imgf = imgfun(k.imgURL);
            //*ifdebug*/log("Poniendo en orden las img functions");
        },
        _getFriendsWithOutActs: function(){
            var r = {};
            for(var k in TuentiProxy.friends){
                r[k] = {};
                copyp(TuentiProxy.friends[k],r[k]);
                r[k]['act'] = undefined;
            }
            return r;
        },
        reorderFriends: function(){
            var temp1 = [];
            for each(var k in TuentiProxy.friends)
                if(k.index !== undefined)
                    temp1.push(k);
            
            temp1.sort(function(a,b){
                return b.index - a.index;
            })

            TuentiProxy.friends = {};
            for each(var k in temp1)
                if(k)TuentiProxy.friends[k.id] = k;
        },
        
	_parseHome: function(endf){
            //*ifdebug*/log("Parseando home");
            txhr({
                url: MISC.home_host + MISC.homefragment,
                migas: MISC.homemigas,
                callback: function(xhr,response){
		    response = preprocess("home0",response);
                    TuentiProxy._parseHome0(endf,response,true);
                }
            });
	},
	_parseHome2: function(endf){
            //*ifdebug*/log("Parseando home (if=)");
            //return endf();
            
            txhr({
                url: TuentiProxy.computeURL(MISC.homeajax),
                callback: function(xhr,response){
		    response = preprocess("home2",response);
                    TuentiProxy._parseHome0(endf,response);
                }
            });
	},
        _parseHome0: function(endf,resp, noajax){
                    var news, green, img,events;
                    //*ifdebug*/log(resp);
                    if(noajax){
                        var config, route, estado,csfr;

                        if(!(config = txtbtwn(resp,STRINGS.CONFIG_ST,STRINGS.CONFIG_EN)))
                            return reportError(701,endf,true);
                        
                        if(!(route = txtbtwn(config,STRINGS.ROUTER_ST,STRINGS.ROUTER_END)))
                            return reportError(703,endf,true);
			
                        if(!(estado = txtbtwn(resp,STRINGS.ESTADO_ST,STRINGS.ESTADO_EN)))
                           estado = "Error.";

                        if(!(img = txtbtwn(resp,REGEXPS.IMG_ST,STRINGS.IMG_EN)))
                            return reportError(705,endf,true);

                        if(!(csfr = txtbtwn(resp,STRINGS.CSFR_ST , STRINGS.CSFR_EN)))
                            return reportError(799,endf,true); //FIXME: 715
                            
                        route += STRINGS.ROUTER_ADD;
                        
                        try{
                            var route = TuentiProxy.router = seval("(" + route + ")");    
                        }catch(e){
                            return reportError(502,endf,true);
                        }
                        
                        
                        if(!route.home_farm && !route.failover_farm )
                            return reportError(501,endf,true);
                        
                        if(!REGEXPS.CSFR.test(csfr))
                                        return reportError(798,endf,true); //FIXME: 715
                            
                        TuentiProxy.myuid = RegExp.$1;  //TODO: declarar
                        TuentiProxy.csfr = csfr;
                        
                        TuentiProxy._estoy = entyties(estado);
                        
                        TuentiProxy.myimg  = {
                            url:  img || "",
                            func: imgfun(img || "")
                        };
                        
                        /*var chat = txtbtwn(resp,  STRINGS['CHAT_INFO_ST']  ,  STRINGS['CHAT_INFO_EN']);
                        if(chat){
                            var cd = seval("(" + chat + ")");
                            cd.port = cd.port||80;
                            cd.resource = cd.resource||"tuenti";
                            if(cd.username && cd.pass && cd.domain && cd.jid){
                            TuentiProxy.chatData = cd;
                            }
                        }*/
                            
                    }
                        
		    
                        
                    if(!(news = txtbtwn(resp,STRINGS.NEWS_ST,STRINGS.NEWS_EN)))
                        reportError(706,endf);
                    
                    if(!(green = txtbtwn(resp,STRINGS.GREEN_ST,STRINGS.GREEN_EN)))
                        reportError(707,endf);
			
                    if(!(events = txtbtwn(resp,STRINGS.EVENT_ST,STRINGS.EVENT_EN)))
                        reportError(708,endf);
                    
		    if(green){
			TuentiProxy._parseNots(green);
		    }
		    if(events){
			TuentiProxy._parseEvents(events);
		    }
		    
		    if(news)news = news.replace("[img","{img")
				       .replace(REGEXPS.EMO,function(str,url){
					    return "[img=" + url + "]";
					});
                    
                    var a = 0;
                    if(news)var acts = news.split(STRINGS.NEWS_SPLIT);
                    newdata = [];
                    if(acts)for(var i=acts.length-1;i>=0;i--){
                        var act = TuentiProxy._parseAct(acts[i]);
                        if(!act)continue;
                        
                        act.index = TuentiProxy.findex++;
                    
                        var f = TuentiProxy.getFriendById(act.id) || {};
                        var na = f.act || {};
                        copyp(act.act || {},na);
                        
                        copyp(act,f);
                        f.act = na;
                    }
                    
                    TuentiProxy.reorderFriends();

                    if(!noajax)return endf();
                    else return TuentiProxy.getAllFriends(endf,config);
                    
        },
        getAllFriends: function(endf,config){
            if(MISC.allfriendsinhome){
                
                if(!config)return reportError(716,endf,true);
                
                var fdata;
                if(!(fdata = txtbtwn(config,STRINGS.FDATA_ST,STRINGS.FDATA_END)))
                    return reportError(702,endf,true);
                
                try{    
                    var friends = seval("(" + fdata + ")");
                    
                    if(!friends || !friends[0] || !friends[0].data)
                        return reportError(719,endf,true);
                        
                    friends = seval("(" + friends[0].data + ")");
                    
                    return TuentiProxy.parseAllFriends(endf,friends);
                    
                }catch(e){
                    return reportError(500,endf,true);
                }
            }else{
                txhr({
                    url: TuentiProxy.computeURL(MISC.allfriendsurl),
                    callback: function(xhr,resp){
			resp = preprocess("allfriends",resp);
                        try{
                            
                            var fdata;
                            if(!(fdata = txtbtwn(resp,STRINGS.ALLFRIENDS_ST,STRINGS.ALLFRIENDS_EN)))
                                return reportError(718,endf,true);
                    
                            var friends = seval("(" + fdata + ")");
                            
                            if(!friends || !friends.results){
                                return reportError(717,endf,true);
                            }
                            
                            return TuentiProxy.parseAllFriends(endf, friends);
                            
                        }catch(e){
                            return reportError(503,endf,true);
                        }
                    },
                    error: function(){
                        return reportError(720,endf,true);
                    }
                });
                
            }
            
            return null;
        },
        parseAllFriends: function(endf,friends){
            
            if(!friends || !friends.results)
                return reportError(709,endf,true);
            
            var m = TuentiProxy.findex = TuentiProxy.findex + friends.results.length + 2;
            for each(var k in friends.results){
                if(k.id && k.string && k.info.avatar_url){
                    copyp(
                        {id: k.id,nick: k.string,fnick: stripAccents(k.string),status: "",imgURL: k.info.avatar_url,imgf: imgfun(k.info.avatar_url),index:--m},
                        TuentiProxy.getNewFriendById(k.id)
                    );
                }
            }
            
            var yo = TuentiProxy.getFriendById(TuentiProxy.myuid);
            if(yo && /\((.*)\)/.test(yo.nick))TuentiProxy.myname = RegExp.$1;
            
            if(TuentiProxy.friends[TuentiProxy.myuid]) delete TuentiProxy.friends[TuentiProxy.myuid];
            
            return endf();
        },
        
	_parseAct:function(act){
                if(act.length < 400)return false;
                		
                if(!REGEXPS.ACT_ESTADO.test(act)){
                    //*ifdebug*/ log("Error en las expresiones regulares de las actualizaciones");
                    return false;
                }
                
                var id = RegExp.$1, estado = RegExp.$2, date = RegExp.$3;
                
                estado = />([^<]*)</.test(estado)?entyties(RegExp.$1 || ""):"";
                
                //*ifdebug*/ log(act);
                
                var acts = {thumbs:[], commentsFromFriends: []};
                if((act = substringfrom(act,STRINGS.ACTS_START))){
                    act.replace(REGEXPS.ACT_THUMB,function(str,url,title,src){
                        acts.thumbs.push({url:fixurl(url),src:src,imgf:imgfun(src),title:entyties(title)});
                        return "";
                    }).replace(REGEXPS.ACT_BLOG,function(str,url,count,title,text){     //fixme: a partir de ahora tenemos el argumento count, hay que cambiar lo de "Un post" por "n post"
                        acts.blog = {url:fixurl(url),title:entyties(title),text:entyties(text), count:count};
                        return "";
                    }).replace(REGEXPS.ACT_FOTOS,function(str,url,count){
                        acts.pictsup = {url:fixurl(url),count:count};
                        return "";
                    }).replace(REGEXPS.ACT_FOTOS_TAG,function(str,url,count){
                        acts.pictstag = {url:fixurl(url),count:count};
                        return "";
                    }).replace(REGEXPS.ACT_FRIEND,function(str,url,count,name){
                        acts.friends = {url:fixurl(url),count:count,name:entyties(name)};
                        return "";
                    }).replace(REGEXPS.ACT_COMEN,function(str,url,count,from){
                        acts.comments = {url:fixurl(url),count:count};
                        return "";
                    }).replace(REGEXPS.ACT_COMEN_PREV_OTHER,function(str,count){
                        acts.commentsFromOthers = count;
                        return "";
                    }).replace(REGEXPS.ACT_COMEN_PREV,function(str,txt,nick){
                        acts.commentsFromFriends.push({txt:entyties(txt),nick:entyties(nick)});
                        return "";
                    });
                }
                
                return {id:id,status:entyties(estado),date:entyties(date),act: acts};
	},
	_parseNots: function(nots){
                var newUpdates = [];
                
                nots.replace(REGEXPS.GREEN,function(str,href,title,clase,text){
                    newUpdates.push({
                        clase: clase,
                        title: entyties(title),
                        url: fixurl(href),
                        txt: entyties(text),
                        displayed: false
                    });
                    return "";
                });
                
                var oldUpdates = TuentiProxy.updates;
                for(var i=0;i<newUpdates.length;i++)
                        for(var k=0;k<oldUpdates.length;k++)
                                if(newUpdates[i].txt === oldUpdates[k].txt){
                                        newUpdates[i].displayed = oldUpdates[k].displayed;
                                        break;
                                }
		
		TuentiProxy.updates = newUpdates;
                TuentiProxy.dispatchEvent("statusChange",{});
                TuentiProxy.dispatchEvent("notifications",{});
		
	},
        _parseEvents: function(resp){
	    
            var ho = resp.indexOf(STRINGS.EVENT_HOY);
            var ma = resp.indexOf(STRINGS.EVENT_MANANA);
            var se = resp.indexOf(STRINGS.EVENT_SEMANA);
            var pr = resp.indexOf(STRINGS.EVENT_PROX);
            
            var hoy = resp.substring(ho,ma);
            var man = (man < 0)?"":resp.substring(ma,se<0?(pr<0?resp.length:pr):se);
            var sem = (se < 0)?"":resp.substring(se,pr<0?resp.length:pr);
            var prx = (pr < 0)?"":resp.substring(pr);
            
            //dump(prx + "\n");
            
            var ev = {hoy:[],'maana':[],semana:[],proximos:[]};
            hoy.replace(REGEXPS.EVENT,function(str,tipo,url,txt,nada,cuando){
                var isCumple = /Birthday/i.test(tipo);
                
                var whn = "";
                if(cuando && />(.*)</.test(cuando)){
                    whn = RegExp.$1;
                }
                var isAllDay = false;
                if(!/([0-9]+):([0-9]+)/.test(whn)){
                    isAllDay = true;
                }
                var date = new Date();
                if(!isAllDay){
                    var horas = RegExp.$1, minutos = RegExp.$2;
                    date.setMinutes(minutos);
                    date.setHours(horas);
                }
                
                ev.hoy.push({birth: isCumple, allday: isAllDay, time: date.getTime(),url: fixurl(url),txt:entyties(txt),cuando:whn});
                return "";
            });
            
            man.replace(REGEXPS.EVENT,function(str,tipo,url,txt,nada,cuando){
                var isCumple = /Birthday/i.test(tipo);
                
                var whn = "";
                if(cuando && />(.*)</.test(cuando)){
                    whn = RegExp.$1;
                }
                
                var isAllDay = false;
                if(!/([0-9]+):([0-9]+)/.test(whn)){
                    isAllDay = true;
                }
                var date = new Date();
                if(!isAllDay){
                    var horas = RegExp.$1, minutos = RegExp.$2;
                    date.setMinutes(minutos);
                    date.setHours(horas);
                }
                date.setDate(date.getDate()+1);
                
                ev['maana'].push({birth: isCumple, allday: isAllDay,time: date.getTime(),url: fixurl(url),txt:entyties(txt),cuando:whn});
                return "";
            });
            
            sem.replace(REGEXPS.EVENT,function(str,tipo,url,txt,nada,cuando){
                var isCumple = /Birthday/i.test(tipo);
                if(cuando && />(.*)</.test(cuando)){
                    var whn = RegExp.$1;
                }else return "";
                if(!/([0-9]+)/.test(whn))return "";
                var dia = RegExp.$1;
                var date = new Date();
                if(dia <= date.getDate())date.setMonth(date.getMonth()+1);
                date.setDate(dia);

                ev.semana.push({birth: isCumple,allday: true ,time: date.getTime(),url: fixurl(url),txt:entyties(txt),cuando:whn});
                return "";
            });
            
            prx.replace(REGEXPS.EVENT,function(str,tipo,url,txt,nada,cuando){
		
                var isCumple = /Birthday/i.test(tipo);
		
                if(cuando && />(.*)</.test(cuando)){
                    whn = RegExp.$1;
                }else return "";
                
                if(!/([0-9]+)( de | )([a-z]+)/i.test(whn))return "";
                var dia = RegExp.$1, mes = RegExp.$3.toLowerCase();
                var date = new Date();
                date.setDate(dia);
                
                var mesi = 'ene,feb,mar,abr,may,jun,jul,ago,sep,oct,nov,dic'.split(",").indexOf(mes);
                if(mesi < 0) return "";
                
                date.setMonth(mesi);

                ev.proximos.push({birth: isCumple,allday: true ,time: date.getTime(),url: fixurl(url),txt:entyties(txt),cuando:whn});
                return "";
            });
            //log(ev);
            
            TuentiProxy.events = ev;
            
        },
	_parseInbox: function(endf){
            //*ifdebug*/log("Parseando inbox");
            //return endf();  //No funciona
            txhr({
                url: TuentiProxy.computeURL(MISC.inboxurl),
                callback: function(xhr,resp){
		    resp = preprocess("inbox",resp);
                    if(!(msgs = txtbtwn(resp,STRINGS.MSG_ST,STRINGS.MSG_EN)))
                        reportError(710,endf);
		    else{
			var inbox = msgs.split(STRINGS.MSG_SPLIT);
			TuentiProxy.inbox = [];
			for(var i=0;i<inbox.length;i++){
			    var a = TuentiProxy._parseMsg(inbox[i]);
			    if(a)TuentiProxy.inbox.push(a);
			}
		    }
                    return endf();
                }
            });
	},
	_parseMsg: function(msg){
                if(msg.length < 400)return false;
                
                if(!REGEXPS.MSG.test(msg))return false;
                
                var name = RegExp.$1, when = RegExp.$3, txt = RegExp.$2;
                
                var r = {id:0, when:entyties(when), txt: entyties(htmltags(txt)) };
                
                r.nuevo = msg.indexOf(STRINGS.MSG_NEW) > 0;
                
                if(REGEXPS.MSG_FULL_URL.test(msg)){
                    r.url = fixurl(RegExp.$1);
                }else{
                    r.url = "";
                }
                
                r.nick = entyties(name);
                r.imgf = imgfun("asdf");
                
                r.imgURL = r.imgf(30);
                return r;
		
	},
        getFullMsg: function(endf,url){
                endf = endf || function(){};
                if(!url || /^\s*$/.test(url))return endf(false);
                
                return txhr({
                    url:url,
                    callback:function(xhr){
                        var resp = xhr.responseText;
                        resp = resp.split("</p><p>").join("\n");
                        var msg = txtbtwn(resp,STRINGS.FMSG_ST,STRINGS.FMSG_EN);
                        if(!msg)endf("error");
                        else endf(htmltags(entyties(msg)));
                    }
                });
        },
	_parseProfile: function(endf){
            //*ifdebug*/log("Parseando profile");
            txhr({
                url: TuentiProxy.computeURL(MISC.profileurl),
                callback: function(xhr,resp){
		    resp = preprocess("profile",resp);
                    //*ifdebug*/ log(resp);
                    var img, com;
                    if(!(img = txtbtwn(resp,STRINGS.PROFILE_IMAGES_ST,STRINGS.PROFILE_IMAGES_END))){
			//log(STRINGS.PROFILE_IMAGES_ST,STRINGS.PROFILE_IMAGES_END);
                        reportError(712,endf);
                    }else{
			img = img.split(STRINGS['PROFILE_IMAGES_SPLIT']);
			
			
			
			TuentiProxy.imagestag = [];
			for(var i=0;i<img.length;i++){
				if(!REGEXPS['IMG_PERFIL'].test(img[i]))continue;
				var url = RegExp.$1, src = RegExp.$2;
				if(!/-([0-9]+)$/.test(url))continue;
				var up = RegExp.$1;
				
				if(up == TuentiProxy.myuid){
					var upl = {nick:''};
					var isYourself = true;
				}else{
					var upl = TuentiProxy.getFriendById(up);
					var isYourself = false;
				}
				TuentiProxy.imagestag[i] = {
					url: fixurl(url),
					imgURL: src,
					imgf: imgfun(src),
					title: "",  //En el nuevo tuenti no hay titulo -_-
					uploader: upl?entyties(upl.nick):"",
					uploadedbyyou: isYourself,
					uploaderisnotafriend: upl?false:true,
					upid:up
				};
			}
		    }
		    
		                        
                    if(!(com = txtbtwn(resp,STRINGS.COM_SPLIT,STRINGS.COM_END))){
                        reportError(712+1,endf);
		    }else{
			com = com.split(STRINGS.COM_SPLIT);
			TuentiProxy.comments = [];
			for(i=0;i<com.length;i++){
			    var co = com[i].replace(STRINGS.COM_REPLACE_FIND,STRINGS.COM_REPLACE_REPLACE);
			    if(!REGEXPS.COM_PERFIL.test(co)){
				//*ifdebug*/ log(co);
				continue;
			    }
			    var id = RegExp.$1,date = RegExp.$2, txt, f = TuentiProxy.getFriendById(id);
			    if(!f)continue;
			    
			    if(!(txt = txtbtwn(co,STRINGS.COM_TXT_ST,STRINGS.COM_TXT_EN)))
				reportError(714,endf);
			    
			    txt = entyties(htmltags(txt));
			    
			    TuentiProxy.comments.push({
				    user_id: id,
				    imgURL: f.imgf(30),
				    imgf: f.imgf,
				    nick: f.nick,
				    when: entyties(date),
				    text: txt,
				    nuevo: false //Al parecer, tuenti marca los comentarios como leeidos cuando visualizamos la pagina, por lo que no se puede saber
				});
			}
		    }
		    return endf();
		}
            });
	},
          /////////////////////////////////////////////////////////////////////
         //                        UTILS                                    //
        /////////////////////////////////////////////////////////////////////
	_getImgFunction: function(imgUrl){
		if(REGEXPS['IMG_SIZE'].test(imgUrl)){
		    var str = imgUrl.replace(REGEXPS['IMG_SIZE'],MISC['urlformat']).split("%SIZE%");
		    return function(width){
			if(width==200){
			    return str.join("200");
			}else if(width == 30){
			    return imgUrl;
			}else{
			    return str.join("600");
			}
		    }
		}else{
		    return function(width){
			return MISC.nophotourl.split("%SIZE%").join(width);
		    }
		}
		
	},
	_convertHTMLEntities: function(html){
		function entity2(str)
		{
		  return String.fromCharCode(str.substring(2,str.length-1));
		}
		function entity(str)
		{
		  var chr = htmlEntities[str.substring(1,str.length-1)];  
		  return chr?String.fromCharCode(chr):str;                 //Si esta en la lista, lo remplazamos por el caracter que corresponda, si no lo dejamos igual
		}
                function longword(word){
                    var urlRegExp = /((http(s?))\:\/\/)([0-9a-zA-Z\-]+\.)+[a-zA-Z]{2,6}(\:[0-9]+)?(\/([\w#!:.?+=&%@~*\';,\-\/\$])*)?/i;
                    if(urlRegExp.test(word)){ //if is a url
                        return word;
                    }
                    var newword = "";
                    for(var i=0;i<word.length; i+=20){
                        newword += word.substring(i,Math.min(i+20,word.length))+" ";
                    }
                    return newword.substring(0,newword.length-1);
                }
                if(!html){
                    return "";
                }
		html = html.replace(/&[a-zA-Z0-9]+;/g,entity);   //Remplace normal entites (&iacute; &trade; etc...);
		html = html.replace(/&#[0-9]+;/g,entity2);   //Remplace the other entities (&#039; &#241; etc...)
		html = html.replace(/\S{20,}/g,longword) //Remplace long words
                
		html = html.replace(/<br[ ]{0,}[\/]?>/gi,"\n");  //Remplace the <br/>'s
		return html;
	},
	_deleteHTMLTags:function(html){
                html = html.replace(/<br[ ]{0,}[\/]?>/gi,"[br]");  //Saltos de linea
                html = html.replace(/<[^>]+\/\s*>/gi,""); //Nodos vacios (<img (...) />)
		function youtube(yurl)
		{
		  return yurl.replace(/^http:\/\/www.youtube.com\/v\/([^& +]+)[&]?[^ ]*$/i,"\nhttp://www.youtube.com/watch?v=$1\n");
		}
                while(/<\s*([a-z]+)[^>]*>.*$/i.test(html)){
                    var etiqueta = RegExp.lastMatch;
                    var left = RegExp.leftContext;
                    var tip = RegExp.$1;
                    
                    //var stXP = new RegExp("<\\s*"  + tip  + "[^>]*>","i");
                    //var enXP = new RegExp("<\\/\\s*"  + tip  + "\\s*>","i");
                    
                    var end = TuentiProxy.findNextR(etiqueta,1,'<'+tip,'</'+tip+'>');
                    var tag = etiqueta.substring(0,end).replace(/[\r\n]/gi,"");
                    
                    //FIXME: Videos y imagenes en el nuevo tuenti
                    var re = "";
                    if(/new SWFObject\(\\"([^"<>?+ ,]+)\\"/.test(tag))
                        re = youtube(RegExp.$1);
                    else if(/<br\/><img src=\\"([^" +]+)\\" [^>]+\/><\/div>/i.test(tag))
                        re = RegExp.$1;
                    else if(/confirmOpenExternLink\('([^']+)'\)/.test(tag))
                        re = RegExp.$1;
                    
                    html = left + re + etiqueta.substring(end,etiqueta.length);
                }
                html = html.replace(/<[a-zA-Z ]+[^<>]*>.*<\/[a-zA-Z ]+>/gi,"").replace(/<[a-zA-Z _:]+\/>/gi,"").replace(/<\/[a-zA-Z ]+>/gi,"").replace(/<.*$/gm,"").replace(/\[br\]/g,"\n");//El resto del html
		return html;
	},
        findNextR: function(str,StartIndex,st,en){
               var lvl = 1;
               var currIndex = StartIndex;
               
               while(true){
                       
                       var ns = str.indexOf(st,currIndex);
                       var ne = str.indexOf(en,currIndex);
                       if(ne<0) return str.length;//no end
                       if(ns<0){
                               lvl--;
                               currIndex = ne + en.length;
                               if(lvl==0)return currIndex;
                               continue;
                       }
                       
                       if(ns>ne){
                               lvl--;
                               currIndex = ne + en.length;
                               if(lvl==0)return currIndex;
                       }else{
                               lvl++;
                               currIndex = ns + st.length;
                       }
               }
        },
          /////////////////////////////////////////////////////////////////////
         //                        IMG UPLOAD                               //
        /////////////////////////////////////////////////////////////////////
	_upListener: null,
	_imgUploading: false,  //TODO add this to desconect function
        /**
         *Start a upload session
         */
	startUploadSession: function(listener){
		if(TuentiProxy._imgUploading)return;
		TuentiProxy._imgUploading = true;
		TuentiProxy._upListener = listener;
	},
        /**
         * Ends a upload session
         */
	endUploadSession: function(response){
		TuentiProxy._imgUploading = false;
		TuentiProxy._upListener = null;
        },
        
        /**
         * Uploads a file to tuenti
         */
	startUploadFile: function(file,iid){
		var bou = TuentiProxy._getBou();
var start =     //TODO: Pasar a los filtros
'--[bou]\n\
Content-Disposition: form-data; name="[FILE]"; filename="[FILE]"\n\
Content-Type: image\n\
\n\
';

var end =
'\n\
--[bou]\n\
Content-Disposition: form-data; name="func"\n\
\n\
addq\n\
--[bou]\n\
Content-Disposition: form-data; name="jv"\n\
\n\
\n\
--[bou]\n\
Content-Disposition: form-data; name="rotate"\n\
\n\
0\n\
--[bou]--\n\
';

                
                start = start.split("[bou]").join(bou).split("[FILE]").join(file.leafName);
                end = end.split("[bou]").join(bou);

		try {
                    var fi = file;
                    
                    //Stream que lee la imagen del disco
                    var st = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
                    st.init(fi,0x01,00004,null);
                    
                    //Stream que lee el stream que lee la imagen del disco
                    var bis = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
                    bis.setInputStream(st);
                    
                    //Stream que mandaremos a traves de la conexion 
                    var pbs = Components.classes["@mozilla.org/storagestream;1"].createInstance(Components.interfaces.nsIStorageStream);
                    pbs.init(1024,10000000,null);
                    
                    //Stream donde escribimos todo
                    var pbbos = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
                    
                    //Unimos el stream "que mandaremos" con que stream "donde escribimos todo"
                    var pbos = pbs.getOutputStream(0);
                    pbbos.setOutputStream(pbos);
                    
                    //Escribir el principio
                    pbbos.writeBytes(start,start.length);
                    
                    //escribir los datos binarios de la imagen (del stream que lee el stream que lee la imagen del disco)
                    var bisbytes = bis.available();
                    pbbos.writeBytes(bis.readBytes(bisbytes),bisbytes);
                    
                    //Escribir el final
                    pbbos.writeBytes(end,end.length);
                    
                    //Cerrarlo todo menos el stream "que mandaremos"
                    bis.close();
                    st.close();
                    pbbos.close();
                    pbos.close();
                    
                    var xhr = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIJSXMLHttpRequest);
                    xhr.open("POST", STRINGS.PHOTO_UPLOAD_URL, true);
                    xhr.setRequestHeader("Content-Type", "multipart/form-data; boundary=" + bou);
                    xhr.setRequestHeader("Referer", STRINGS.PHOTO_UPLOAD_REF);
                    xhr.setRequestHeader("Content-Length", start.length);
                    
                    xhr.onreadystatechange = function(){
                        if(xhr.readyState != 4)return false;
                        if(xhr.status != 200)
                            return TuentiProxy._upListener.onUploadError(iid,3);
                            
                        TuentiProxy._upListener.onUploadComplete(iid);
                        var res = xhr.responseText;
                        //*ifdebug*/dump(res);
                        var idst = res.indexOf(STRINGS.PHOTO_ID_ST) + STRINGS.PHOTO_ID_ST.length;
                        var iden = res.indexOf(STRINGS.PHOTO_ID_EN , idst);
                                        
                        if(idst < 0||iden <0)
                            return TuentiProxy._upListener.onUploadError(iid,2);
                                
                        var id = res.substring(idst,iden);
                        
                        if(!/^[0-9]+$/.test(id) || id == 0)
                            return TuentiProxy._upListener.onUploadError(iid,1);
                            
                        var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
                        var intentos = 0;
                        var xhrc = null;
                        var event = {
                            notify: function() {
                                
                                if(++intentos > 5){
                                    timer.cancel();
                                    TuentiProxy._upListener.onUploadError(iid,6);
                                    return;
                                }
                                //*ifdebug*/dump("abortando...");
                                if(xhrc)xhrc.abort();
                                
                                xhrc = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIJSXMLHttpRequest);
                                xhrc.open("POST", STRINGS.PHOTO_UPLOAD_URL, true);
                                xhrc.onreadystatechange = function(){
                                    //*ifdebug*/dump("RSC!: rs:" + xhrc.readyState + "  st: " + xhrc.status + "\n");
                                    if(xhrc.readyState != 4 || xhrc.status != 200)return;
                                    
                                    var res = xhrc.responseText;
                                    
                                    //*ifdebug*/dump(res + "\n");
                                    
                                    var idst = res.indexOf(STRINGS.PHOTO_ID_ST) + STRINGS.PHOTO_ID_ST.length;
                                    var iden = res.indexOf(STRINGS.PHOTO_ID_EN , idst);
                                    
                                    //*ifdebug*/dump(idst + " - " + iden + "\n");
                                    
                                    if(idst <= STRINGS.PHOTO_ID_ST.length || iden <= idst)return;
                                        
                                    var id2 = res.substring(idst,iden);
                                    //*ifdebug*/dump(" :D 1 " + id2 + " \n");
                                    if(!/^[0-9\-]+$/.test(id2))return;
                                    //*ifdebug*/dump(" :D 2 \n");
                                    timer.cancel();
                                    TuentiProxy._upListener.onUploadEnd(iid,id2);
                                };
                                
                                xhrc.setRequestHeader("Referer", STRINGS.PHOTO_UPLOAD_REF);
                                xhrc.setRequestHeader("Content-Type","application/x-www-form-urlencoded");
                                //*ifdebug*/dump("Conectando...\n");
                                xhrc.send(STRINGS.PHOTO_CONF_SEND.split("[ID]").join(id));
                            }
                        };
                        return timer.initWithCallback(event, 4000 ,Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
                    };
                    xhr.onerror = function(){TuentiProxy._upListener.onUploadError(iid,4);};
                    xhr.send(pbs.newInputStream(0));
			
		}catch(e){
                        //*ifdebug*/dump(e);
			TuentiProxy._upListener.onUploadError(iid,5);
		}

	},
        tagImage: function(id,tags,title){ //TODO: pasar a los filtros
            var en = encodeURIComponent;
            
            var post = en("item_ids[]") + "=" + en(id) + "&";
            post += "csrf=" + TuentiProxy.csfr + "&";
            post += "collection_key=" + en(id) + "&";
            post += "moving_tag=false&";
            
            
            var json = "";
            for each(var k in tags){
                var x = Math.round(k.px), y = Math.round(k.py)
                json += ',{"user_id":"'+k.id+'","x":'+x+',"y":'+y+'}';
            }
            json = en("[" + json.substring(1) + "]");
            
            post += "tags_added=" + json  + "&" ;
            post += "tags_removed=" + en("[]") + "&";
            post += "photo_title=" + en(title);
            
            var url = TuentiProxy.computeURL("?m=Photoedit&func=process_edit_photo&coming_from="+id+"&offset=1&ajax=1&store=0&ajax_target=canvas");
            
            txhr({
                url:url,
                method: "POST",
                plain: post
            });
        },
          /////////////////////////////////////////////////////////////////////
         //                           FIND UTILS                            //
        /////////////////////////////////////////////////////////////////////
	init: function(d){
		for(var i=0;i<d.length;i++){
			this.friends[d[i][0]] = {id:d[i][0],nick:d[i][1],fnick:stripAccents(d[i][1])};
		}
	},
	getNewFriendById : function(id){
		var f = TuentiProxy.friends[id];
                if(f)return f;
                TuentiProxy.friends[id] = {};
                return TuentiProxy.friends[id];
	},
	getFriendById : function(id){
		return TuentiProxy.friends[id];
	},
	getFriendByName : function(name){
		name = stripAccents(name);
		for each(var k in TuentiProxy.friends)
			if(k.fnick == name)
				return k;
		return null;
	},	
	findFriend: function(str){
		str = stripAccents(str);
		var res = [];
		for each(var k in TuentiProxy.friends)
			if(k.fnick && k.fnick.indexOf(str) > -1)
				res.push(k.nick);
		return res;
	},
	findFriend2: function(str){
		str = stripAccents(str);
		var res = [];
		for each(var k in TuentiProxy.friends)
			if(k.fnick.indexOf(str) > -1){
				var cst = k.fnick.indexOf(str);
				var nme = k.nick;
				res.push(nme.substring(0,cst) + "|" + nme.substring(cst,cst + str.length -1) + "|" + nme.substring(cst + str.length -1));
			}
		return res;
	}
};


var getUniqueId=(function(){
		var A=Math.floor(Math.random()*1000000+1);
		return function(){
			return A++
		}
	})();

/*****************
 * NET FUNCTIONS *
 *****************/

var xhr_stack = [];

function txhr(stuff){
    if(!stuff.url) return;
    var post     = stuff.method == "POST";
    var url      = stuff.url;
    var headers  = stuff.headers    || {};
    var parms    = stuff.parms      || {};
    var callback = stuff.callback   || function(){};
    var plain    = stuff.plain      || "";
    var galleta  = stuff.galleta    || false;
    var migas    = stuff.migas      || false;
    var retry    = stuff.retry      !== false;
    var error    = stuff.error      || function(){TuentiProxy.dispatchEvent("error",{errorcde:204,critical:true});};
    var nocatch  = stuff.nocatch    || false;
    var noreffer = stuff.noreffer   || false;
    stuff.runs   = stuff.runs+1     || 0;
    
    var xhr = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIJSXMLHttpRequest);
    xhr.mozBackgroundRequest = true;
    
    if(!stuff.index)stuff.index = xhr_stack.push(xhr)-1;
    var index = stuff.index;
    
    if(stuff.runs > MAX_RETRIES){
        error();
        delete xhr_stack[index];
        return;
    }
    
    var timer = new Scheduler(function(){
        //*ifdebug*/log("Connection Timeout, data:",stuff);
        xhr.tuentiAborted = true;
        xhr.abort();
        if(retry)txhr(stuff);
        else error();
        },CONECTION_TIMEOUT);
    
    xhr.aborttimer = function(){
        timer.stop();
    }
    
    xhr.onreadystatechange = function(){
            if(xhr.readyState != 4)return;
            timer.stop();
            if(xhr.tuentiAborted)return;
            if(xhr.status == 200 || xhr.status == 302){     //A veces tuenti usa el codigo 302, pero la pagina carga bien
                timer.stop();
                if(nocatch)
                    callback(xhr,xhr.responseText);
                else{
                    var txt = xhr.responseText;
                    if(txt.indexOf(STRINGS.DISCONECTED_MSG)>=0){   //have been disconected T.T
                        //*ifdebug*/log("reconectando...");
                        TuentiProxy._login(function(){  //Re connect
                            txhr(stuff);   //Retry
                        });
                    }else   //All ok!
                        callback(xhr,xhr.responseText);
                }
            }else if(xhr.status >= 500 && xhr.status < 600){  //Server error
                //*ifdebug*/log("Error del servidor, data:",stuff);
                if(retry)txhr(stuff);//Retry
                else error();
            }else{                                            //4xx, 3xx, no connection, etc...
                //*ifdebug*/log("Error raro, data:",stuff);
                if(retry)txhr(stuff);//Retry
                else error();
            }
    }
    var paramsStr = "";
    for (var key in parms)
        paramsStr += "&" + key + "=" + parms[key];
    paramsStr = paramsStr.substring(1) + plain;
    url += (post?"":paramsStr);
    
    // Open the XHR
    if (post) {
        xhr.open("POST", url, true);
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    } else {
        xhr.open("GET", url, true);
    }
    
    if(stuff.xml)xhr.overrideMimeType("text/xml");
    
    for(var k in headers)
        xhr.setRequestHeader(k,headers[k]);
    
    if(!noreffer)xhr.setRequestHeader('Referer',"http://www.tuenti.com/");
    
    if(galleta || migas)
        new CookieMonster(xhr,migas!=false,migas!=false?migas:galleta);
    
    xhr.send(post?paramsStr:null);
}
//Cancel all conections
function calcellAllConectios(){
    //*ifdebug*/log("abortando conexiones...");
    for(var k in xhr_stack){
        xhr_stack[k].abort();
        xhr_stack[k].aborttimer();
        xhr_stack[k].tuentiAborted = true;  //Para que no se reintente
        delete xhr_stack[k];
    }
}



/****************************************************************
 *                          UTILS                               *
 ****************************************************************/


/*
 *Funcion que retorna la zona horaria <==> function that return the timezone
 */
/**
 * This function will return what timezone the user is in as an offset from GMT in hours
 * (i.e. if the user is in Madrid (GMT +100) this would return 1)
 * Note that the function assumes that Daylight Savings Time is NOT observed in the month of 
 * January anywhere in the world Tuenti cares about.
 */
function get_time_zone() {						
	var date_now = new Date();
	var date_winter_local = new Date(date_now.getFullYear(), 0, 1, 0, 0, 0, 0);			   
	var temp = date_winter_local.toGMTString();
	var date_winter_gmt = new Date(temp.substring(0, temp.lastIndexOf(" ")-1));
	//timestamps are stored in terms of milliseconds, hence the 1000 * 60 * 60 to convert
	//it to hours.
	var hoursDiffStdTime = (date_winter_local - date_winter_gmt) / (1000 * 60 * 60);						   
	return hoursDiffStdTime;			    
}


/*
 * Algoritmo md5 <==> md5 algorithm
 * nesesario para el login   <==> needed for login
 */
function hex_md5(str){
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";
    var result = {};
    var data = converter.convertToByteArray(str, result);
    var ch = Components.classes["@mozilla.org/security/hash;1"].createInstance(Components.interfaces.nsICryptoHash);
    ch.init(ch.MD5);
    ch.update(data, data.length);
    var hash = ch.finish(false);
    function toHexString(charCode){return ("0" + charCode.toString(16)).slice(-2);}
    return [toHexString(hash.charCodeAt(i)) for (i in hash)].join("");
}


Components.utils.import("resource://tuenti/cookieMonster.jsm"); 


/**
 *Function that eliminates the accents from a string and trims it. Ej. Input " Camin" Return "camion"
 *@param text {string} The string to convert
 *@returns {string} The string with no accents
 */
const accents         = ['','','','','','',''];
const no_accents      = ['a','e','i','o','u','u','o'];
function stripAccents(text){
        text = text.toLowerCase().replace(/^\s*/,'').replace(/\s*$/,'');
        for (var letter in accents){
                text = text.replace(new RegExp(accents[letter],"ig"),no_accents[letter]);
        }
        return " " + text.replace(/[^a-z ]/gm,'');
}

function copyp(from,to){
    for(var k in from)
        to[k] = from[k];
}

function txtbtwn(pajar,start,end){
    var stt = indexOfX(pajar,start,0);
    if(stt[0]<0)return false;
    var st = stt[0]+stt[1].length;
    var en = indexOfX(pajar,end,st);
    if(en[0]<0)return false;
    return pajar.substring(st,en[0]);
}

var indexOfX = function(str,Regxp,index){
    if(typeof Regxp == 'string')
        return [str.indexOf(Regxp,index),Regxp];
    else{
        var r = [str.substring(index).search(Regxp),RegExp.lastMatch];
	if(r[0]>=0){
	    r[0] += index
	}
	return r;
    }
}

function substringfrom(pajar,aguja){
    var st = pajar.indexOf(aguja);
    if(st<0)return false;
    return pajar.substring(st);
}
function fixurl(url){
    return "http://www." + MISC.domain + "/" + url.split("&amp;").join("&");
}
//Para hacer mas pequeo, lejible y rapido el codigo
const entyties = TuentiProxy._convertHTMLEntities;
const htmltags = TuentiProxy._deleteHTMLTags;
const imgfun = TuentiProxy._getImgFunction;

function reportError(code,func,critical){
    TuentiProxy.dispatchEvent("error",{errorcde:code||100,critical:!!critical});   //100 for unknown error
    if(func && typeof func == 'function')func();
}

function seval(cde){
    return Components.utils.evalInSandbox(cde,sBox);
}


const prefix = "TFox: ";
var log = (function(){
    //var tipo = Components.classes["@mozilla.org/fuel/application;1"].getService(Components.interfaces.fuelIApplication).prefs.getValue("extensions.tuentifox.api.logger","none");
    var tipo ="console";
    var exlogger;
    if (tipo == "dump")
         exlogger = function log(){
	    var nativeJSON = Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON);
            var nativeJSON = Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON);
            Array['prototype'].forEach.call(arguments,function(k){
                if(typeof k == typeof {} || typeof k == typeof [])
                    dump(prefix + nativeJSON.encode(k) + "\n");
                else
                    dump(prefix + k + "\n");
            });
        }
    else if(tipo == "console"){
	var nativeJSON = Components.classes["@mozilla.org/dom/json;1"].createInstance(Components.interfaces.nsIJSON);
        var app = Components.classes["@mozilla.org/fuel/application;1"].getService(Components.interfaces.fuelIApplication);
        exlogger = function log(){
            Array['prototype'].forEach.call(arguments,function(k){
                if(typeof k == typeof {} || typeof k == typeof [])
                    app.console.log(prefix + nativeJSON.encode(k));
                else
                    app.console.log(prefix + k);
            });
        }
    }else
        exlogger = function(){};
	
    return function(){
	TuentiProxy.dispatchEvent("log",{cosas:arguments});
	exlogger.apply(this,arguments);
    }
})();





